Package: princurve
Version: 2.1.6
Title: Fit a Principal Curve in Arbitrary Dimension
Authors@R: c(
    person("Trevor", "Hastie", email = "trevor@research.att.com", role = "aut"),
    person("Andreas", "Weingessel", email = "Andreas.Weingessel@ci.tuwien.ac.at", role = "aut"),
    person(
      "Kurt", "Hornik", 
      email = "kurt.hornik@wu.ac.at", 
      role = "aut",
      comment = c(ORCID = "0000-0003-4198-9911")
    ),
    person(
      "Henrik", "Bengtsson", 
      email = "henrikb@braju.com", 
      role = "ctb",
      comment = c(github = "HenrikBengtsson")
    ),
    person(
      "Robrecht", "Cannoodt", 
      email = "rcannood@gmail.com", 
      role = c("aut", "cre"),
      comment = c(ORCID = "0000-0003-3641-729X", github = "rcannood")
    ))
Description: Fitting a principal curve to a data matrix in arbitrary dimensions. 
  Hastie and Stuetzle (1989) <doi:10.2307/2289936>.
License: GPL-2
Encoding: UTF-8
Depends: R (>= 3.0)
Imports: stats, graphics, grDevices, Rcpp
Suggests: devtools, testthat
LinkingTo: Rcpp
NeedsCompilation: yes
RoxygenNote: 7.1.1
URL: https://github.com/rcannood/princurve
BugReports: https://github.com/rcannood/princurve/issues
Collate: 'RcppExports.R' 'bias_correct_curve.R' 'deprecated.R'
        'package.R' 'periodic_lowess.R' 'smoother_functions.R'
        'principal_curve.R' 'start_circle.R'
Packaged: 2021-01-17 18:16:00 UTC; rcannood
Author: Trevor Hastie [aut],
  Andreas Weingessel [aut],
  Kurt Hornik [aut] (<https://orcid.org/0000-0003-4198-9911>),
  Henrik Bengtsson [ctb] (HenrikBengtsson),
  Robrecht Cannoodt [aut, cre] (<https://orcid.org/0000-0003-3641-729X>,
    rcannood)
Maintainer: Robrecht Cannoodt <rcannood@gmail.com>
Repository: CRAN
Date/Publication: 2021-01-18 16:40:06 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-26 10:43:02 UTC; unix
Archs: princurve.so.dSYM
