# Generated by roxygen2: do not edit by hand

S3method(lines,principal.curve)
S3method(lines,principal_curve)
S3method(plot,principal.curve)
S3method(plot,principal_curve)
S3method(points,principal.curve)
S3method(points,principal_curve)
export(get.lam)
export(principal.curve)
export(principal_curve)
export(project_to_curve)
export(whiskers)
import(Rcpp)
importFrom(grDevices,extendrange)
importFrom(graphics,lines)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(graphics,segments)
importFrom(stats,approx)
importFrom(stats,lowess)
importFrom(stats,predict)
importFrom(stats,smooth.spline)
importFrom(stats,var)
useDynLib(princurve)
